From 29d4aafce065aef02402bc0c9928dafece9216fc Mon Sep 17 00:00:00 2001
From: Keerthy <j-keerthy@ti.com>
Date: Thu, 10 Nov 2022 11:34:36 +0530
Subject: [PATCH] Enable MCU ESM

The patch adds MCU ESM & enables configuring multiple MCU instances in R5 SPL.
This enables propagating the main domain ESM events to MCU ESM that in turn
enables the Main domain watchdogs triggering a MCU domain / SoC reset.

Signed-off-by: Keerthy <j-keerthy@ti.com>
---
 arch/arm/dts/k3-j721e-r5-sk.dts |  7 +++++++
 arch/arm/mach-k3/j721e_init.c   | 25 +++++--------------------
 drivers/misc/esm_pmic.c         | 30 ++++++++++++++++++++++++++++++
 3 files changed, 42 insertions(+), 20 deletions(-)

diff --git a/arch/arm/dts/k3-j721e-r5-sk.dts b/arch/arm/dts/k3-j721e-r5-sk.dts
index 03a1bba9fa..e92606c2d9 100644
--- a/arch/arm/dts/k3-j721e-r5-sk.dts
+++ b/arch/arm/dts/k3-j721e-r5-sk.dts
@@ -219,6 +219,13 @@
 				<&mcu_secproxy 23>;
 		u-boot,dm-spl;
 	};
+
+	mcu_esm: esm@40800000 {
+		compatible = "ti,j721e-esm";
+		reg = <0x0 0x40800000 0x0 0x1000>;
+		ti,esm-pins = <84>, <85>, <95>;
+		u-boot,dm-spl;
+	};
 };
 
 &cbass_main {
diff --git a/arch/arm/mach-k3/j721e_init.c b/arch/arm/mach-k3/j721e_init.c
index e9b78d4bcf..c1d703d58a 100644
--- a/arch/arm/mach-k3/j721e_init.c
+++ b/arch/arm/mach-k3/j721e_init.c
@@ -574,42 +574,27 @@ void board_init_f(ulong dummy)
 		do_board_detect();
 
 #if defined(CONFIG_CPU_V7R) && defined(CONFIG_K3_AVS0)
-	if (board_ti_k3_is("J721EX-PM1-SOM")) {
-		offset = fdt_node_offset_by_compatible(gd->fdt_blob, -1,
-						       "ti,am654-vtm");
-		val = fdt_getprop_u32_default_node(gd->fdt_blob, offset, 0,
-						   "som1-supply-2", dflt);
-		do_fixup_by_compat_u32((void *)gd->fdt_blob, "ti,am654-vtm",
-				       "vdd-supply-2", val, 0);
-	}
-
 	ret = uclass_get_device_by_driver(UCLASS_MISC, DM_GET_DRIVER(k3_avs),
 					  &dev);
 	if (ret)
 		printf("AVS init failed: %d\n", ret);
 #endif
 
-#ifdef CONFIG_ESM_K3
-	if (board_ti_k3_is("J721EX-PM2-SOM") ||
-	    board_ti_k3_is("J7200X-PM2-SOM")) {
+#if defined(CONFIG_CPU_V7R)
 		ret = uclass_get_device_by_driver(UCLASS_MISC,
 						  DM_GET_DRIVER(k3_esm), &dev);
 		if (ret)
 			printf("MISC init failed: %d\n", ret);
-	}
-#endif
-
-#ifdef CONFIG_ESM_PMIC
-	if (board_ti_k3_is("J721EX-PM2-SOM") ||
-	    board_ti_k3_is("J7200X-PM2-SOM")) {
+		ret = uclass_get_device_by_name(UCLASS_MISC,
+						  "esm@700000", &dev);
+		if (ret)
+			printf("MISC init 1 failed: %d\n", ret);
 		ret = uclass_get_device_by_driver(UCLASS_MISC,
 						  DM_GET_DRIVER(pmic_esm),
 						  &dev);
 		if (ret)
 			printf("ESM PMIC init failed: %d\n", ret);
-	}
 #endif
-
 #if defined(CONFIG_K3_J721E_DDRSS)
 	ret = uclass_get_device(UCLASS_RAM, 0, &dev);
 	if (ret)
diff --git a/drivers/misc/esm_pmic.c b/drivers/misc/esm_pmic.c
index a195dc5eb1..e9393057dc 100644
--- a/drivers/misc/esm_pmic.c
+++ b/drivers/misc/esm_pmic.c
@@ -26,6 +26,17 @@
 #define ESM_MCU_EN		BIT(6)
 #define ESM_MCU_ENDRV		BIT(5)
 
+#define MASK_ESM_REG	0x59
+
+#define ESM_SOC_START_REG	0x98
+
+#define ESM_SOC_START		BIT(0)
+
+#define ESM_SOC_MODE_CFG_REG	0x9b
+
+#define ESM_SOC_EN		BIT(6)
+#define ESM_SOC_ENDRV		BIT(5)
+
 /**
  * pmic_esm_probe: configures and enables PMIC ESM functionality
  *
@@ -54,6 +65,25 @@ static int pmic_esm_probe(struct udevice *dev)
 		return ret;
 	}
 
+	ret = pmic_reg_write(dev->parent, MASK_ESM_REG, 0x0);
+	if (ret) {
+		dev_err(dev, "unmasking ESM interrupts failed: %d\n", ret);
+		return ret;
+	}
+
+	ret = pmic_reg_write(dev->parent, ESM_SOC_MODE_CFG_REG,
+			     ESM_MCU_EN | ESM_MCU_ENDRV);
+	if (ret) {
+		dev_err(dev, "starting SOC Mode failed: %d\n", ret);
+		return ret;
+	}
+
+	ret = pmic_reg_write(dev->parent, ESM_SOC_START_REG, ESM_SOC_START);
+	if (ret) {
+		dev_err(dev, "starting SOC ESM failed: %d\n", ret);
+		return ret;
+	}
+
 	return 0;
 }
 
-- 
2.17.1

